@@ -3,9 +3,11 @@ import six
 import time
 import numpy as np
 import pandas as pd
+from warnings import catch_warnings, filterwarnings, resetwarnings, warn
+from cobra.exceptions import OptimizationError
 from dexom_python.imat_functions import imat
 from dexom_python.result_functions import write_solution
-from dexom_python.model_functions import load_reaction_weights, read_model, check_model_options, DEFAULT_VALUES
+from dexom_python.model_functions import load_reaction_weights, read_model, check_model_options, DEFAULT_VALUES, check_threshold_tolerance
 from dexom_python.enum_functions.enumeration import EnumSolution, create_enum_variables, read_prev_sol
 from dexom_python.enum_functions.icut_functions import create_icut_constraint
 from dexom_python.enum_functions.maxdist_functions import create_maxdist_constraint, create_maxdist_objective
@@ -47,6 +49,7 @@ def diversity_enum(model, reaction_weights, prev_sol=None, eps=DEFAULT_VALUES['e
     solution: an EnumSolution object
     stats: a pandas.DataFrame containing the number of selected reactions and runtime of each iteration
     """
+    check_threshold_tolerance(model=model, epsilon=eps, threshold=thr)
     if prev_sol is None:
         prev_sol = imat(model, reaction_weights, epsilon=eps, threshold=thr, full=full)
     else:
@@ -78,25 +81,46 @@ def diversity_enum(model, reaction_weights, prev_sol=None, eps=DEFAULT_VALUES['e
         selected_recs.append(i)
         objective = create_maxdist_objective(model, tempweights, prev_sol, prev_sol_bin, full=full)
         model.objective = objective
-        try:
-            t2 = time.perf_counter()
-            print('time before optimizing in iteration '+str(idx)+':', t2-t0)
-            with model:
-                prev_sol = model.optimize()
-            prev_sol_bin = (np.abs(prev_sol.fluxes) >= thr-tol).values.astype(int)
-            all_solutions.append(prev_sol)
-            all_binary.append(prev_sol_bin)
-            if save:
-                write_solution(model, prev_sol, thr,
-                               filename=out_path+'_solution_'+time.strftime('%Y%m%d-%H%M%S')+'.csv')
-            t1 = time.perf_counter()
-            print('time for optimizing in iteration ' + str(idx) + ':', t1 - t2)
-            times.append(t1 - t0)
-        except:
-            print('An error occured in iteration %i of dexom, no solution was returned' % idx)
-            times.append(-1)
-            prev_sol = all_solutions[-1]
-
+        t2 = time.perf_counter()
+        print('time before optimizing in iteration ' + str(idx) + ':', t2 - t0)
+        with catch_warnings():
+            filterwarnings('error')
+            try:
+                with model:
+                    prev_sol = model.optimize()
+                prev_sol_bin = (np.abs(prev_sol.fluxes) >= thr-tol).values.astype(int)
+                all_solutions.append(prev_sol)
+                all_binary.append(prev_sol_bin)
+                if save:
+                    write_solution(model, prev_sol, thr,
+                                   filename=out_path+'_solution_'+time.strftime('%Y%m%d-%H%M%S')+'.csv')
+                t1 = time.perf_counter()
+                print('time for optimizing in iteration ' + str(idx) + ':', t1 - t2)
+                times.append(t1 - t0)
+            except UserWarning as w:
+                resetwarnings()
+                times.append(-1)
+                prev_sol = all_solutions[-1]
+                if 'time_limit' in str(w):
+                    print('The solver has reached the timelimit in iteration %i. If this happens frequently, there may '
+                          'be too many constraints in the model. Alternatively, you can try modifying solver '
+                          'parameters such as the feasibility tolerance or the MIP gap tolerance.' % idx)
+                    warn('Solver status is "time_limit" in iteration %i' % idx)
+                elif 'infeasible' in str(w):
+                    print('The solver has encountered an infeasible optimization in iteration %i. If this happens '
+                          'frequently, there may be a problem with the starting solution. Alternatively, you can try '
+                          'modifying solver parameters such as the feasibility tolerance or the MIP gap tolerance.'
+                          % idx)
+                    warn('Solver status is "infeasible" in iteration %i' % idx)
+                else:
+                    print('An unexpected error has occured during the solver call in iteration %i.' % idx)
+                    warn(w)
+            except OptimizationError as e:
+                resetwarnings()
+                times.append(-1)
+                prev_sol = all_solutions[-1]
+                print('An unexpected error has occured during the solver call in iteration %i.' % idx)
+                warn(str(e), UserWarning)
     model.solver.remove([const for const in icut_constraints if const in model.solver.constraints])
     model.solver.remove(opt_const)
     solution = EnumSolution(all_solutions, all_binary, all_solutions[0].objective_value)
