@@ -4,9 +4,11 @@ import time
 import pandas as pd
 import numpy as np
 from symengine import Add, sympify
+from warnings import warn, catch_warnings, filterwarnings, resetwarnings
+from cobra.exceptions import OptimizationError
 from dexom_python.enum_functions.icut_functions import create_icut_constraint
 from dexom_python.imat_functions import imat
-from dexom_python.model_functions import load_reaction_weights, read_model, check_model_options, DEFAULT_VALUES
+from dexom_python.model_functions import load_reaction_weights, read_model, check_model_options, DEFAULT_VALUES, check_threshold_tolerance
 from dexom_python.enum_functions.enumeration import EnumSolution, create_enum_variables, read_prev_sol
 
 
@@ -115,6 +117,7 @@ def maxdist(model, reaction_weights, prev_sol=None, eps=DEFAULT_VALUES['epsilon'
     -------
     solution: EnumSolution object
     """
+    check_threshold_tolerance(model=model, epsilon=eps, threshold=thr)
     if prev_sol is None:
         prev_sol = imat(model, reaction_weights, epsilon=eps, threshold=thr, full=full)
     else:
@@ -128,26 +131,48 @@ def maxdist(model, reaction_weights, prev_sol=None, eps=DEFAULT_VALUES['epsilon'
     opt_const = create_maxdist_constraint(model, reaction_weights, prev_sol, obj_tol,
                                           name='maxdist_optimality', full=full)
     model.solver.add(opt_const)
-    for i in range(maxiter):
+    for idx in range(1, maxiter+1):
         t0 = time.perf_counter()
         if icut:
             # adding the icut constraint to prevent the algorithm from finding the same solutions
-            const = create_icut_constraint(model, reaction_weights, thr, prev_sol, name='icut_'+str(i), full=full)
+            const = create_icut_constraint(model, reaction_weights, thr, prev_sol, name='icut_'+str(idx), full=full)
             model.solver.add(const)
             icut_constraints.append(const)
         # defining the objective: minimize the number of overlapping ones and zeros
         objective = create_maxdist_objective(model, reaction_weights, prev_sol, prev_sol_bin, only_ones, full)
         model.objective = objective
-        try:
-            with model:
-                prev_sol = model.optimize()
-            prev_sol_bin = (np.abs(prev_sol.fluxes) >= thr-tol).values.astype(int)
-            all_solutions.append(prev_sol)
-            all_binary.append(prev_sol_bin)
-        except:
-            print('An error occured in iter %i of maxdist' % (i+1))
-        t1 = time.perf_counter()
-        print('time for iteration '+str(i+1)+': ', t1-t0)
+        with catch_warnings():
+            filterwarnings('error')
+            try:
+                with model:
+                    prev_sol = model.optimize()
+                prev_sol_bin = (np.abs(prev_sol.fluxes) >= thr-tol).values.astype(int)
+                all_solutions.append(prev_sol)
+                all_binary.append(prev_sol_bin)
+                t1 = time.perf_counter()
+                print('time for iteration ' + str(idx) + ':', t1 - t0)
+            except UserWarning as w:
+                resetwarnings()
+                prev_sol = all_solutions[-1]
+                if 'time_limit' in str(w):
+                    print('The solver has reached the timelimit in iteration %i. If this happens frequently, there may '
+                          'be too many constraints in the model. Alternatively, you can try modifying solver '
+                          'parameters such as the feasibility tolerance or the MIP gap tolerance.' % idx)
+                    warn('Solver status is "time_limit" in iteration %i' % idx)
+                elif 'infeasible' in str(w):
+                    print('The solver has encountered an infeasible optimization in iteration %i. If this happens '
+                          'frequently, there may be a problem with the starting solution. Alternatively, you can try '
+                          'modifying solver parameters such as the feasibility tolerance or the MIP gap tolerance.'
+                          % idx)
+                    warn('Solver status is "infeasible" in iteration %i' % idx)
+                else:
+                    print('An unexpected error has occured during the solver call in iteration %i.' % idx)
+                    warn(w)
+            except OptimizationError as e:
+                resetwarnings()
+                prev_sol = all_solutions[-1]
+                print('An unexpected error has occured during the solver call in iteration %i.' % idx)
+                warn(str(e), UserWarning)
     model.solver.remove([const for const in icut_constraints if const in model.solver.constraints])
     model.solver.remove(opt_const)
     solution = EnumSolution(all_solutions, all_binary, all_solutions[0].objective_value)
