@@ -30,11 +30,15 @@ def main():
 
     rxn_sol = rxn_enum(model=model, rxn_list=[], prev_sol=imat_solution, reaction_weights=reaction_weights, eps=eps,
                        thr=thr, obj_tol=obj_tol)
-    pd.DataFrame(rxn_sol.unique_binary).to_csv('toy_models/small4M_rxnenum_solutions.csv')
+    uniques = pd.DataFrame(rxn_sol.unique_binary)
+    uniques.columns = [r.id for r in model.reactions]
+    uniques.to_csv('toy_models/small4M_rxnenum_solutions.csv')
 
-    div_sol = diversity_enum(model=model, prev_sol=imat_solution, reaction_weights=reaction_weights, eps=eps, thr=thr,
-                             obj_tol=obj_tol, maxiter=maxiter, dist_anneal=dist_anneal,
-                             out_path='toy_models/small4M_divenum')
+    div_sol, div_res = diversity_enum(model=model, prev_sol=imat_solution, reaction_weights=reaction_weights, eps=eps,
+                                      thr=thr, obj_tol=obj_tol, maxiter=maxiter, dist_anneal=dist_anneal)
+    div_res.to_csv('toy_models/small4M_divenum_results.csv')
+    sol = pd.DataFrame(div_sol.binary, columns=[r.id for r in model.reactions])
+    sol.to_csv('toy_models/small4M_divenum_solutions.csv')
     return True
 
 
